@echo off
rem echo on
setlocal

::===============================================
::
::	Updated version (2019-12-17)
::	- Fixed an issue with $ in foldername to index.
::	  The $ wasn't shown correctly in Everything's windowtitle
::	- Fixed an issue when IndexThis was in a folder containing
::	  & in it's name.
::	- Installation in foldernames with "^"  is no longer allowed.
::	  The ^ is too "toxic"
::	- Fixed an issue with indexiong a folder with "()" in it's name.
::	- Use NTFS indexing for NTFS volumes that are not bootvolume
::	   without the service being needed.
::	- Added an option to use a different icon in the context menu
::
::===============================================
::
::	Updated version (2018-10-25)
::	- Use a different detection method of NTFS
::		filesystem that doesnt need admin rights
::	- Check if Everything service is running 
::		(needed for NTFS indexing)
::
::===============================================


::_______________________________________________
::
::	SETTINGS
::_______________________________________________
::

::-----------------------------------------------
::	Location of Everything.exe (without quotes)
::-----------------------------------------------
	rem set EverythingEXE=C:\Tools\Everything\Everything.exe
	set EverythingEXE=C:\Program Files\Everything\Everything.exe

::-----------------------------------------------
::	Icon to use in context menu (without quotes)
::-----------------------------------------------

::	If not defined: use Everything.exe icon
::
::	Some examples:
::	set ICON=c:\path to\My Favorite.ico
::	set ICON=%SystemRoot%\system32\compstui.dll,-64040
::	set ICON=c:\program files\everything\everything.exe
::	set ICON=C:\test\TestDAta\Index This\Everything Yellow.ico
	set ICON=

::_______________________________________________
::
::	INIT
::_______________________________________________
::

::	$ has special meaning in Everything window title; escape it.	
	set TITLEPATH=%1
	set TITLEPATH=%TITLEPATH:$=$$%



::_______________________________________________
::
::	ACTION!
::_______________________________________________
::

	If [%1]==[] goto :CONTEXT

::	Stop running temp-index
	start /wait "" "%EverythingEXE%" -nodb -instance TEMPINDEX  -config "%TEMP%\DeleteThis.ini" -quit


::	Create new base INI  (%TEMP%\IndexThis.ini)
	(echo [Everything]
	echo run_as_admin=0
	echo run_in_background=0
	echo show_in_taskbar=1
	echo show_tray_icon=0
	echo minimize_to_tray=0
	echo check_for_updates_on_startup=0
	echo auto_include_fixed_volumes=0
	echo auto_include_removable_volumes=0
	echo auto_include_fixed_refs_volumes=0
	echo auto_include_removable_refs_volumes=0
	echo hide_on_close=0
	rem	echo window_title_format=$i %1
	echo window_title_format=$i %TITLEPATH%
	) > "%TEMP%\IndexThis.ini"




::	Figure out if NTFS indexing is possible; otherwise Folder Indexing
::	Assume Folder Indexing until  proven otherwise.
	
::	Used logic:
::	IF ntfs =0 ==> use Folder indexing
::	ELSE (meaning ntfs=1) IF bootvol = 0 ==> NTFS indexiing
::	ELSE (meaning nfs=1 and bootvol=1) IF service running ==> NTFS indexiing

	set INDEXMETHOD=FOLDER
	
::	Check this volume for filesystem and bootvolume
	for /f "usebackq" %%x in (`wmic.exe volume where DriveLetter^="%~d1" get FileSystem^,bootvolume /value`) Do set %%x>nul

	IF /i [%FileSystem%] neq [NTFS] (
		rem Keep using folder indexing
	) ELSE IF /i [%bootvolume%] neq [TRUE] (
		set INDEXMETHOD=NTFS
	) ELSE (
		sc.exe query everything| findstr /i "running" >nul && set INDEXMETHOD=NTFS
	)



::	Show indexing method to use
	echo Use %INDEXMETHOD% indexing for %1

::	Add filesystem-specific entries to the INI
	call :%INDEXMETHOD% %%1
::	INI complete; start Everything ...
	start "" "%EverythingEXE%" -nodb -instance TEMPINDEX -config "%TEMP%\IndexThis.ini"

goto :EOF


::===============================================
::
::	SUBROUTINES
::
::===============================================
::


::-----------------------------------------------
:NTFS
::-----------------------------------------------

:: Get VolumeID and convert \ to \\
	for /f "usebackq delims= " %%x in (`mountvol %~d1 /L`) Do set VOLUMEID=%%x
	set VOLUMEID=%VOLUMEID:\=\\%

	set INDEX=%1

	:: Write NTFS settings to INI
	:: %INDEX% separate because of possible ()
	(
		echo ntfs_volume_guids="%VOLUMEID%"
		echo ntfs_volume_paths="%~d1"
		echo ntfs_volume_roots=""
		echo ntfs_volume_includes=1
		echo service_pipe_name=\\.\PIPE\Everything Service
	) >> "%TEMP%\IndexThis.ini"
 	echo ntfs_volume_include_onlys="%INDEX%">> "%TEMP%\IndexThis.ini"
goto :EOF



::-----------------------------------------------
:FOLDER
::-----------------------------------------------

	set INDEX=%1
	set INDEX=%INDEX:\=\\%
	echo folders=%INDEX% >> "%TEMP%\IndexThis.ini"
goto :EOF





::-----------------------------------------------
:CONTEXT
::-----------------------------------------------

	color 1b
	echo.
	echo.
	echo.	[I] Install IndexThis context menu
	echo.	[R] Remove  IndexThis context menu
	echo.	[Q] Quit
	echo.

	choice /c IRQ /N /M "Enter your choice : "
	call :Option%ERRORLEVEL%
	echo.
	pause
goto :EOF



::-----------------------------------------------
:Option1   ::Install context menu
::-----------------------------------------------

::	Check and escape special characters for the registry
	set REGFOLDER="%~dpnx0"
	
	:: Foldernames with "^" and "%" are not allowed; EXIT
	set "NOGO="
	echo.
	echo %REGFOLDER%|findstr /i /C:"^" && set NOGO=1
	echo %REGFOLDER%|findstr /i /C:"%%" && set NOGO=1
	If defined NOGO (
		echo.
		echo.	Foldernames with "^" or "%%" are not allowed.
		echo.	Please install %~nx0 in a different folder
		echo.
		goto :EOF
	)
	
	set "REGFOLDER=%REGFOLDER:&=^^^&%"


::	Check Icon configuration
	If not DEFINED ICON set ICON=%EXE%
	For /f "delims=," %%x in ("%ICON%") do if not exist "%%x" set ERROR=Check ICON=%ICON%

	If Defined ERROR (
		echo.
		echo.  Icon misconfiguration
		echo.  Check the SETTINGS section of the script
		echo.
		echo.  ^( Script: %~f0 ^)
		echo.
		goto :EOF
	)
  
::-----------------------------------------------
::	Add contextmenu to registry for DIRECTORIES
::-----------------------------------------------

	::	Add command to execute
	REG.exe ADD HKCU\Software\Classes\Directory\Shell\IndexThis\Command /ve /d "%REGFOLDER:"=\"% \"%%1\"" -f

	::	Add icon to contexmenu
	REG.exe ADD HKCU\Software\Classes\Directory\Shell\IndexThis /v Icon /d "\"%ICON%\"" -f

	
::-----------------------------------------------
::	Add contextmenu to registry for DRIVES
::-----------------------------------------------

	::	Add command to execute
	REG ADD HKCU\Software\Classes\Drive\Shell\IndexThis\Command /ve /d "%REGFOLDER:"=\"% \"%%1\"" -f

	::	Add icon to contexmenu
	REG ADD HKCU\Software\Classes\Drive\Shell\IndexThis /v Icon /d "\"%ICON%\"" -f

goto :EOF




::-----------------------------------------------
:Option2  ::Remove context menu
::-----------------------------------------------
	::	Remove context menu
	REG DELETE HKCU\Software\Classes\Directory\Shell\IndexThis /f
	REG DELETE HKCU\Software\Classes\Drive\Shell\IndexThis /f
goto :EOF


::-----------------------------------------------
:Option3  ::Quit
::-----------------------------------------------
	::	Do nothing
goto :EOF

